package com.example.tdl.web;

import com.example.tdl.domain.dto.CommFeedback;
import com.example.tdl.domain.vo.*;
import com.example.tdl.entity.AlarmLog;
import com.example.tdl.service.AlarmLogService;
import com.example.tdl.service.CircuitService;
import com.example.tdl.service.CompanyService;
import com.example.tdl.service.GatewayService;
import com.example.tdl.service.redis.TokenRedisService;
import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.Map;


@RestController
@RequestMapping("/alarmLog")
public class AlarmLogController {
    private Logger logger= LoggerFactory.getLogger(AlarmLogController.class);

    Gson gson= new Gson();

    CommFeedback fb=new CommFeedback();

    @Autowired
    private AlarmLogService alarmLogServcie;

    @Autowired
    private GatewayService gatewayService;

    @Autowired
    private CircuitService circuitService;

    @Autowired
    private TokenRedisService tokenRedisService;

    @Autowired
    private CompanyService companyService;

    //获取报警信息
    @ApiOperation(value = "获取报警信息",notes = "获取报警信息" +
                "   transportationNo:运输编号" +
                "   description:报警内容" +
                "   startCity:启运城市" +
                "   endCity:目的城市" +
                "   startTime:出发时间（时间戳）" +
                "   endTime:结束时间（时间戳）" +
            "返回值:      transportationNo:运输编号" +
            "   cargoNo:货号" +
            "   startCity:启运城市" +
            "   endCity:目的城市" +
            "   description:报警内容" +
            "   alarmTime:报警时间" )
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @PostMapping("/getAlarm")
    public Object getAlarm(@RequestBody AlarmTermVo alarmTermVo ,HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" + token);
        UserRedisVo user = gson.fromJson(datum, UserRedisVo.class);
        if(StringUtils.isEmpty(alarmTermVo.getTransportationNo())){
            alarmTermVo.setTransportationNo(null);
        }
        if(StringUtils.isEmpty(alarmTermVo.getDescription())){
            alarmTermVo.setDescription(null);
        }
        if(StringUtils.isEmpty(alarmTermVo.getEndCity())){
            alarmTermVo.setEndCity(null);
        }
        if(StringUtils.isEmpty(alarmTermVo.getStartCity())){
            alarmTermVo.setStartCity(null);
        }
        if(alarmTermVo.getStartTime()==null){
            alarmTermVo.setStartTime(null);
        }
        if(alarmTermVo.getDescription()==null){
            alarmTermVo.setEndTime(null);
        }
        alarmTermVo.setCompanyName(user.getCompanyName());
        return  alarmLogServcie.getAlarm(alarmTermVo);
    }

    //获取最新报警
    @ApiOperation(value = "获取报警信息",notes = "获取报警信息" +
            "返回值:  transportationNo:运输编号" +
            "data:报警数据" +
            "description:报警内容" +
            "alarmTime:报警时间" )
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @PostMapping("/getRealTimeAlarm")
    public Object getRealTimeAlarm(HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" + token);
        UserRedisVo user = gson.fromJson(datum, UserRedisVo.class);
        return  alarmLogServcie.getRealTimeAlarm(user.getCompanyName());
    }


    @ApiOperation(value = "获取报警信息",notes = "获取报警信息" +
            "返回值:  userName:用户名" +
            "password:密码" )
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @GetMapping("/getAlarmMessage")
    public String getAlarmMessage(HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" + token);
        UserRedisVo userRedisVo = gson.fromJson(datum,UserRedisVo.class);
        ResultCompanyVo companyVo =  companyService.getByCompanyName(userRedisVo.getCompanyName());
        return  gson.toJson(new MqttUser("ugen","ugen",companyVo.getCompanyNo()));
    }

    //新增报警消息
    @ApiOperation(value = "增加报警",notes = "增加报警")
    @RequestMapping(value = "/addAlarmLog",method = RequestMethod.POST)
    public String addAlarmLog(@RequestBody String addAlarmLog){
        logger.info(addAlarmLog);
        AddAlarmLogVo addAlarmLogVo = gson.fromJson(addAlarmLog,AddAlarmLogVo.class);
        logger.info(System.currentTimeMillis()+"--------"+gson.toJson(addAlarmLogVo));
        if(StringUtils.isEmpty(addAlarmLogVo.getGsn())){
            fb.setCode(0);
            fb.setMessage("网关序列号不能为空");
        }
        if(StringUtils.isEmpty(addAlarmLogVo.getGtype())){
            fb.setCode(0);
            fb.setMessage("网关类型不能为空");
        }
        if(addAlarmLogVo.getAlarm()==null){
            fb.setCode(0);
            fb.setMessage("报警数据不能为空");
        }
        if(addAlarmLogVo.getTs() == null){
            fb.setCode(0);
            fb.setMessage("时间不能为空");
        }
        Map<Integer,String> map = new HashMap<>();
        map.put(0,"正常");
        map.put(1,"等于最大值");
        map.put(2,"不等于最大值");
        map.put(3,"过高");
        map.put(4,"过低");
        map.put(5,"未知");
        //根据gateway的信息获取线路信息
        String transportationNo =  gatewayService.getTransportationNo(addAlarmLogVo.getGsn(),addAlarmLogVo.getGtype());
        //获取终点，起点
        ResultCircuitVo resultCircuitVo = circuitService.getCircuit(transportationNo);
        for(int i = 0;i<addAlarmLogVo.getAlarm().size();i++){
            String classify="";
            String desp = "";
            if(addAlarmLogVo.getAlarm().get(i).getAlias().contains("T")){
                desp = "温度" +map.get(addAlarmLogVo.getAlarm().get(i).getAlarmno());
                classify = "1";
            } else if(addAlarmLogVo.getAlarm().get(i).getDesp().contains("h")){
                desp = "湿度" +map.get(addAlarmLogVo.getAlarm().get(i).getAlarmno());
                classify = "2";
            } else if(addAlarmLogVo.getAlarm().get(i).getDesp().contains("a0")){
                desp = "震动" +map.get(addAlarmLogVo.getAlarm().get(i).getAlarmno());
                classify = "3";
            } else if(addAlarmLogVo.getAlarm().get(i).getDesp().contains("ta")){
                desp = "倾角" +map.get(addAlarmLogVo.getAlarm().get(i).getAlarmno());
                classify = "4";
            } else if(addAlarmLogVo.getAlarm().get(i).getDesp().contains("离线")){
                desp = addAlarmLogVo.getAlarm().get(i).getTdlsn()+"离线";
                classify = "6";
            }
            AlarmLog alarmLog = new AlarmLog(transportationNo,resultCircuitVo.getStartCity(),resultCircuitVo.getEndCity(),addAlarmLogVo.getAlarm().get(i).getTdlsn().replace("TDL-","").trim(),desp,addAlarmLogVo.getAlarm().get(i).getValue(),addAlarmLogVo.getAlarm().get(i).getTs()*1000,classify);
            int a = alarmLogServcie.addAlarmLog(alarmLog);
            if(a == 0){
                fb.setCode(0);
                fb.setMessage("添加失败");
            }else{
                fb.setCode(1);
                fb.setMessage("添加成功");
            }
        }
        return gson.toJson(fb);
    }

}
