package com.example.tdl.web;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.example.tdl.domain.dto.CommFeedback;
import com.example.tdl.domain.vo.AddCompanyVo;
import com.example.tdl.domain.vo.UpdateCompanyVo;
import com.example.tdl.service.CompanyService;
import com.example.tdl.service.RoleService;
import com.example.tdl.service.redis.TokenRedisService;
import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;


@RestController
@RequestMapping("/company")
public class CompanyController {

    private Logger logger = LoggerFactory.getLogger(CompanyController.class);

    CommFeedback fb=new CommFeedback();

    Gson gson=new Gson();

    @Autowired
    private CompanyService companyService;

    @Autowired
    private TokenRedisService tokenRedisService;


    //查询所有公司信息
    @ApiOperation(value = "查询所有公司信息",notes = "查询所有公司信息,返回值说明：" +
            "   companyNo:公司编号，" +
            "   companyName:公司名，" +
            "   country:国家，" +
            "   city:城市，" +
            "   addressDetail:具体地址，" +
            "   tel:电话号码，" +
            "   manager:负责人")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/getAll",method = RequestMethod.GET)
    public Object getAll(){
        return companyService.getAll();
    }

    //添加公司信息
    @ApiOperation(value = "添加公司信息",notes = "添加公司信息,companyNo公司编号不用传，需要传的值：" +
            "   companyName:公司名，" +
            "   country:国家，" +
            "   city:城市，" +
            "   addressDetail:具体地址，" +
            "   tel:电话号码，" +
            "   manager:负责人")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/addCompany",method = RequestMethod.POST)
    public Object addCompany(@RequestBody AddCompanyVo addCompanyVo){
        if (StringUtils.isEmpty(addCompanyVo.getCompanyName())){
            fb.setCode(0);
            fb.setMessage("添加的公司名");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addCompanyVo.getCountry())){
            fb.setCode(0);
            fb.setMessage("添加的公司所在国家");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addCompanyVo.getCity())){
            fb.setCode(0);
            fb.setMessage("添加的公司所在城市");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addCompanyVo.getAddressDetail())){
            fb.setCode(0);
            fb.setMessage("添加的公司所在具体地址不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addCompanyVo.getTel())){
            fb.setCode(0);
            fb.setMessage("添加的公司联系电话不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addCompanyVo.getManager())){
            fb.setCode(0);
            fb.setMessage("添加的公司负责人不能为空");
            return gson.toJson(fb);
        }
        if( !addCompanyVo.getTel().matches("^1[34578]\\d{9}$")){
            fb.setCode(0);
            fb.setMessage("手机号不规范");
            return gson.toJson(fb);
        }
        if (companyService.getByCompanyName(addCompanyVo.getCompanyName())!=null){
            fb.setCode(0);
            fb.setMessage("添加的公司名不能重复");
            return gson.toJson(fb);
        }
        Date day=new Date();
        SimpleDateFormat df = new SimpleDateFormat("yyyyMMdd");
        String companyNo="TM"+df.format(day);
        String companyNos=companyNo+((int)(Math.random()*(9999-1000+1))+1000);
        if (companyService.getByCompanyNo(companyNos)!=null) {
            while (companyService.getByCompanyNo(companyNos) != null) {
                companyNos = companyNo + ((int) (Math.random() * (9999 - 1000 + 1)) + 1000);
            }
        }
        addCompanyVo.setCompanyNo(companyNos);
        int a = companyService.addCompany(addCompanyVo);
        if (a>0){
            fb.setCode(1);
            fb.setMessage("添加公司成功");
        }else{
            fb.setCode(0);
            fb.setMessage("添加公司失败");
        }
        return gson.toJson(fb);
    }

    //修改公司信息
    @ApiOperation(value = "修改公司信息",notes = "修改公司信息，需要传的值：" +
            "   companyNo:需要修改的公司编号，" +
            "   companyName:修改后的公司名，" +
            "   country:国家，" +
            "   city:城市，" +
            "   addressDetail:具体地址，" +
            "   tel:电话，" +
            "   manager:负责人")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/updateCompany",method = RequestMethod.POST)
    public Object updateCompany(@RequestBody UpdateCompanyVo updateCompanyVo){
        if (StringUtils.isEmpty(updateCompanyVo.getCompanyNo())){
            fb.setCode(0);
            fb.setMessage("需要修改的公司的公司编号");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateCompanyVo.getCompanyName())){
            fb.setCode(0);
            fb.setMessage("修改的公司名");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateCompanyVo.getCountry())){
            fb.setCode(0);
            fb.setMessage("修改的公司所在国家");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateCompanyVo.getCity())){
            fb.setCode(0);
            fb.setMessage("修改的公司所在城市");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateCompanyVo.getAddressDetail())){
            fb.setCode(0);
            fb.setMessage("修改的公司所在具体地址不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateCompanyVo.getTel())){
            fb.setCode(0);
            fb.setMessage("修改的公司联系电话不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateCompanyVo.getManager())){
            fb.setCode(0);
            fb.setMessage("修改的公司负责人不能为空");
            return gson.toJson(fb);
        }
        if( !updateCompanyVo.getTel().matches("^1[34578]\\d{9}$")){
            fb.setCode(0);
            fb.setMessage("手机号不规范");
            return gson.toJson(fb);
        }
        if (companyService.getByCompanyName(updateCompanyVo.getCompanyName())!=null&&!companyService.getByCompanyNo(updateCompanyVo.getCompanyNo()).getCompanyName().equals(updateCompanyVo.getCompanyName())){
            fb.setCode(0);
            fb.setMessage("修改的公司名不能重复");
            return gson.toJson(fb);
        }
        int a = companyService.updateCompany(updateCompanyVo);
        if (a>0){
            fb.setCode(1);
            fb.setMessage("修改公司成功");
        }else{
            fb.setCode(0);
            fb.setMessage("修改公司失败");
        }
        return gson.toJson(fb);
    }

    //删除公司
    @ApiOperation(value = "删除公司",notes = "删除公司,传值说明：companyName:公司编号")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/delCompany",method = RequestMethod.POST)
    public Object delCompany(@RequestBody String companyNo){
        JSONObject jsonObject= JSON.parseObject(companyNo);
        companyNo=(String) jsonObject.get("companyNo");
        Map<Object,Object> map=new HashMap<>();
        map.put("companyNo",companyNo);
        companyService.delCompany(map);
        Map<Object,Object> msg=new HashMap<>();
        msg.put("msg",map.get("msg"));
        if (msg.get("msg").equals("删除公司成功")){
            fb.setCode(1);
            fb.setMessage("删除公司成功");
        }else{
            fb.setCode(0);
            fb.setMessage(msg.get("msg").toString());
        }
        return gson.toJson(fb);
    }

}
